package de.hft.stuttgart.citydoctor2.zip;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

public class CityGmlZipEntryFile implements AutoCloseable {

    private final ZipFile zip;
    private final ZipEntry zipEntry;
    private boolean closed = false;
    private static final long MB = 1024 * 1024L;

    public CityGmlZipEntryFile(CityGmlZipEntry entry) throws IOException {
        CityGmlZipArchive archive = entry.getArchive();
        zip = new ZipFile(archive.getArchivePath().toFile());
        zipEntry = zip.getEntry(entry.getFileName());
    }

    public InputStream getInputStream() throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        return zip.getInputStream(zipEntry);
    }

    public long getFileSize() throws IOException {
        if (closed){
            throw new IOException("Stream closed");
        }
        if (zipEntry.getSize() != -1){
            return (long) Math.ceil((double) zipEntry.getSize() / MB);
        }
        long bytes = 0;
        InputStream is = this.getInputStream();
        for(int i = is.read(); i != -1; i=is.read()) {
            bytes++;
        }
        return (long) Math.ceil((double) bytes / MB);
    }

    @Override
    public void close() throws IOException {
        zip.close();
        closed = true;
    }
}
