const nrOfPages = 7;
const itemsPerPage = 20;

var token = window.localStorage.getItem('token');
var loadedValidationPlan = false;

var fileProgress = $('#file-progress');
var invalidFile = $('#invalidFile');
var invalidConfigFile = $('#invalidConfigFile');
var onlyErrors = false;
var currentValidationId = null;
var currentModelId = null;

Dropzone.options.addFileZone = {
	init : function() {
		this.on("sending", function(file, xhr, formData) {
			invalidFile.slideUp();
			fileProgress.attr('aria-valuenow', '0');
			fileProgress.width('0%');
		});
		this.on('complete', function(file) {
			this.removeFile(file);
		});
		this.on('error', function(file, errorMessage, xhr) {
			$('#invalidFileText').text(errorMessage);
			invalidFile.slideDown();
		});
		this.on('success', function(file, response) {
			refreshModels(token);
		});
	},
	parallelUploads : 1,
	createImageThumbnails : false,
	maxFiles : 1,
	headers : {
		'Authorization' : 'Bearer ' + token
	},
	thumbnail : function(file, dataUrl) {
		// Display the image in your file.previewElement
	},
	uploadprogress : function(file, progress, bytesSent) {
		// Display the progress
		fileProgress.attr('aria-valuenow', progress);
		fileProgress.width(progress + '%');
	},
	addedfile : function(file) {
	},
	acceptedFiles : '.xml, .gml, .zip',
};

$(document).ready(function() {
	createToggleListener();
	createFileNameListener();
	setupUploadButton(token);
	setupBackButton();
	setupFilterEventListener();
	refreshModels(token);
	addModalListener();
});

function setupFilterEventListener() {
	$('#only_errors').change(function() {
		onlyErrors = $(this).prop('checked');
		loadBuildings(currentModelId, 0, currentValidationId);
	});
}

function addModalListener() {
	$('#validation_plan_modal').on('show.bs.modal', function (event) {
		  var button = $(event.relatedTarget);
		  var modelId = button.data('id');
		  if (!loadedValidationPlan) {
			  loadedValidationPlan = true;
			  $.ajax({
				  method : 'GET',
				  url : '/CityDoctorWebService/rest/checks',
				  success : function(response) {
					  var geometricChecksDiv = $('#geometric_checks');
					  response.geometricChecks.forEach(function (check, i) {
						  var checkDiv = document.createElement('div');
						  geometricChecksDiv.append(checkDiv);
						  
  						  var input = document.createElement('input');
  						  checkDiv.append(input);
  						  input.setAttribute('type', 'checkbox');
  						  input.setAttribute('checked', '');
  						  input.setAttribute('id', 'check' + i);
  						  input.setAttribute('data-toggle', 'toggle');
  						  input.setAttribute('data-onstyle', 'success');
  						  input.setAttribute('data-offstyle', 'danger');
  						  input.setAttribute('data-size', 'sm');
  						  $(input).bootstrapToggle();
  						 
  						  var label = document.createElement('label');
  						  checkDiv.append(label);
  						  label.setAttribute('class', 'ml-1');
  						  label.setAttribute('for', 'check' + i);
  						  label.append(check.name);
  						  
  						  if (check.parameters.length != 0) {
  							  var groupDiv = document.createElement('div');
  							  checkDiv.append(groupDiv);
  							  groupDiv.setAttribute('class', 'ml-3 list-group list-group-flush');
  							  var itemDiv = document.createElement('div');
  	  						  groupDiv.append(itemDiv);
  	  						  itemDiv.setAttribute('class', 'list-group-item');
							  
  							  check.parameters.forEach(function (parameter, i) {
  	  							  
  	  							  var inputGroupDiv = document.createElement('div');
  	  							  itemDiv.append(inputGroupDiv);
  	  							  inputGroupDiv.setAttribute('class', 'input-group');
  	  							  
  	  							  var prependDiv = document.createElement('div');
  	  							  inputGroupDiv.append(prependDiv);
  	  							  prependDiv.setAttribute('class', 'input-group-prepend');
  	  							  
  	  							  var span = document.createElement('span');
  	  							  prependDiv.append(span);
  	  							  span.setAttribute('class', 'input-group-text');
  	  							  span.append(parameter.name);
  	  							  
  	  							  var input = document.createElement('input');
  	  							  inputGroupDiv.append(input);
  	  							  input.setAttribute('type', 'text');
  	  							  input.setAttribute('class', 'form-control');
  	  							  input.setAttribute('value', parameter.defaultValue);
  							  });
  						  }
					  });
				  },
				  complete : function(xhr, status) {
					  if (xhr.status !== 200) {
						  loadedValidationPlan = false;
						  if (xhr.status === 500) {
							  document.location.href = 'internal_server_error.html';
						  }
					  }
				  }
			  });
		  }
	});
}

function logoutClick() {
	$.ajax({
		method : 'POST',
		url : '/CityDoctorWebService/rest/logout',
		beforeSend : function(xhr) {
			xhr.setRequestHeader('Authorization', 'Bearer ' + token);
		},
		success : function(response) {
			document.location.href = 'index.html';
		},
		complete : function(xhr, status) {
			if (xhr.status == 403) {
				document.location.href = 'index.html';
			} else if (xhr.status == 500) {
				document.location.href = 'internal_server_error.html';
			}
		}
	});
}

function setupBackButton() {
	$('#backButton').click(function() {
		$('#modelsPage').slideToggle();
		$('#viewPage').slideToggle();
		stopRender();
	});
}

function refreshModels(token) {
	var container = $('#models');
	container.empty();
	$.ajax({
		method : 'GET',
		url : '/CityDoctorWebService/rest/models',
		beforeSend : function(xhr) {
			xhr.setRequestHeader('Authorization', 'Bearer ' + token);
		},
		success : function(response) {
			createModelBars(response, container);
		},
		complete : function(xhr, status) {
			if (xhr.status == 403) {
				document.location.href = 'index.html';
			} else if (xhr.status == 500) {
				document.location.href = 'internal_server_error.html';
			}
		}
	});
}

function createModelBars(models, container) {
	models.forEach(function(model, i) {
		var div = document.createElement('div');
		div.setAttribute('class', 'row justify-content-center')
		container.append(div);
		
		var colDiv = document.createElement('div');
		div.append(colDiv);
		colDiv.setAttribute('class', 'col-md-8');
		
		var cardDiv = document.createElement('div');
		colDiv.append(cardDiv);
		cardDiv.setAttribute('class', 'card border border-secondary');
		
		var modelName = 'model' + i;
		var headingId = 'headingModel' + i;
		
		var heading = document.createElement('h5');
		cardDiv.append(heading);
		heading.setAttribute('class', 'card-title card-header');
		heading.setAttribute('data-toggle', 'collapse');
		heading.setAttribute('data-target', '#' + modelName);
		heading.setAttribute('aria-expanded', 'false');
		heading.setAttribute('aria-controls', modelName);
		heading.setAttribute('id', headingId);
		
		var headingButton = document.createElement('button');
		heading.append(headingButton);
		headingButton.setAttribute('class', 'btn btn-link text-dark');
		headingButton.setAttribute('data-toggle', 'collapse');
		headingButton.setAttribute('data-target', '#' + modelName);
		headingButton.setAttribute('aria-expanded', 'false');
		headingButton.setAttribute('aria-controls', modelName);
		headingButton.innerHTML = model.name;
		
		var collapseDiv = document.createElement('div');
		cardDiv.append(collapseDiv);
		collapseDiv.setAttribute('id', modelName);
		collapseDiv.setAttribute('class', 'collapse');
		collapseDiv.setAttribute('aria-labelledby', headingId);
		
		var cardBodyDiv = document.createElement('div');
		collapseDiv.append(cardBodyDiv);
		cardBodyDiv.setAttribute('class', 'card-body');
		
		var deleteButton = document.createElement('button');
		cardBodyDiv.append(deleteButton);
		deleteButton.setAttribute('class', 'btn btn-danger float-right');
		deleteButton.append('Delete');
		deleteButton.addEventListener('click', function(e) {
			$.ajax({
				method : 'DELETE',
				url : '/CityDoctorWebService/rest/models/' + model.id,
				beforeSend : function(xhr) {
					xhr.setRequestHeader('Authorization', 'Bearer ' + token);
				},
				success : function(response) {
					div.parentNode.removeChild(div);
				},
				complete : function(xhr, status) {
					if (xhr.status == 403) {
						document.location.href = 'index.html';
					} else if (xhr.status == 500) {
						document.location.href = 'internal_server_error.html';
					}
				}
			});
		});
		
		var badge = document.createElement('span');
		heading.append(badge);
		if (model.status == 'OK') {
			badge.setAttribute('class', 'badge badge-success float-right');
			badge.innerHTML = 'Parsing Successful';
			addOKParsingControls(cardBodyDiv, model);
		} else if (model.status == 'ERROR') {
			badge.setAttribute('class', 'badge badge-danger float-right');
			badge.innerHTML = 'Parsing Failed';
		} else if (model.status == 'PARSING') {
			badge.setAttribute('class', 'badge badge-warning float-right');
			badge.innerHTML = 'Parsing in Progress';
			window.setTimeout(function() {
				modelStatusListener(model, cardBodyDiv, badge);
			}, 1000);
		}
	});
}

var modelStatusListener = function (model, root, badge) {
	$.ajax({
		method : 'GET',
		url : '/CityDoctorWebService/rest/models/' + model.id + '/status',
		beforeSend : function(xhr) {
			xhr.setRequestHeader('Authorization', 'Bearer ' + token);
		},
		success : function(response) {
			if (response.status == 'OK') {
				badge.setAttribute('class', 'badge badge-success float-right');
				badge.innerHTML = 'Parsing Successful';
				addOKParsingControls(root, model);
			} else if (response.status == 'PARSING') {
				window.setTimeout(function() {
					modelStatusListener(model, root, badge);
				}, 1000);
			} else if (response.status == 'ERROR') {
				badge.setAttribute('class', 'badge badge-danger float-right');
				badge.innerHTML = 'Parsing Failed';
			}
		},
		complete : function(xhr, status) {
			if (xhr.status == 403) {
				document.location.href = 'index.html';
			} else if (xhr.status == 500) {
				document.location.href = 'internal_server_error.html';
			}
		}
	});
}

var validationStatusListener = function (validation, root, badge, model) {
	$.ajax({
		method : 'GET',
		url : '/CityDoctorWebService/rest/models/validations/' + validation.id + '/status',
		beforeSend : function(xhr) {
			xhr.setRequestHeader('Authorization', 'Bearer ' + token);
		},
		success : function(response) {
			if (response.status == 'PARSING') {
				badge.setAttribute('class', 'badge badge-warning float-right');
				badge.innerHTML = 'Parsing';
				window.setTimeout(function() {
					validationStatusListener(validation, root, badge, model);
				}, 1000);
			} else if (response.status == 'VALIDATING') {
				badge.setAttribute('class', 'badge badge-warning float-right');
				badge.innerHTML = 'Validating';
				window.setTimeout(function() {
					validationStatusListener(validation, root, badge, model);
				}, 1000);
			} else if (response.status == 'OK') {
				badge.setAttribute('class', 'badge badge-success float-right');
				badge.innerHTML = 'Validation Successful';
				addOKValidationControls(root, model, validation);
			} else if (response.status == 'ERROR') {
				badge.setAttribute('class', 'badge badge-danger float-right');
				badge.innerHTML = 'Validation Failed';
			}
		},
		complete : function(xhr, status) {
			if (xhr.status == 403) {
				document.location.href = 'index.html';
			} else if (xhr.status == 500) {
				document.location.href = 'internal_server_error.html';
			}
		}
	});
}

function searchInputEvent(input) {
	console.log(input.value);
}

function downloadFile(url) {
	var xhr = new XMLHttpRequest();
	xhr.open('GET', url, true);
	xhr.responseType = 'arraybuffer';
	xhr.onload = function () {
	    if (this.status === 200) {
	        var filename = "";
	        var disposition = xhr.getResponseHeader('Content-Disposition');
	        if (disposition && disposition.indexOf('attachment') !== -1) {
	            var filenameRegex = /filename[^;=\n]*=((['"]).*?\2|[^;\n]*)/;
	            var matches = filenameRegex.exec(disposition);
	            if (matches != null && matches[1]) filename = matches[1].replace(/['"]/g, '');
	        }
	        var type = xhr.getResponseHeader('Content-Type');

	        var blob;
	        if (typeof File === 'function') {
	            try {
	                blob = new File([this.response], filename, { type: type });
	            } catch (e) { /* Edge */ }
	        }
	        if (typeof blob === 'undefined') {
	            blob = new Blob([this.response], { type: type });
	        }

	        if (typeof window.navigator.msSaveBlob !== 'undefined') {
	            // IE workaround for "HTML7007: One or more blob URLs were revoked by closing the blob for which they were created. These URLs will no longer resolve as the data backing the URL has been freed."
	            window.navigator.msSaveBlob(blob, filename);
	        } else {
	            var URL = window.URL || window.webkitURL;
	            var downloadUrl = URL.createObjectURL(blob);

	            if (filename) {
	                // use HTML5 a[download] attribute to specify filename
	                var a = document.createElement("a");
	                // safari doesn't support this yet
	                if (typeof a.download === 'undefined') {
	                    window.location = downloadUrl;
	                } else {
	                    a.href = downloadUrl;
	                    a.download = filename;
	                    document.body.appendChild(a);
	                    a.click();
	                }
	            } else {
	                window.location = downloadUrl;
	            }

	            setTimeout(function () { URL.revokeObjectURL(downloadUrl); }, 100); // cleanup
	        }
	    }
	};
	xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
	xhr.setRequestHeader('Authorization', 'Bearer ' + token);
	xhr.send();
}

function addOKParsingControls(root, model) {
	var viewButton = document.createElement('button');
	root.append(viewButton);
	viewButton.setAttribute('class', 'btn btn-primary')
	viewButton.innerHTML = 'View';
	viewButton.onclick = function(e) {
		$('#modelsPage').slideToggle();
		$('#viewPage').slideToggle();
		loadBuildings(model.id, 0, null);
	};
	
	var validationHeading = document.createElement('h5');
	root.append(validationHeading);
	validationHeading.setAttribute('class', 'mt-3');
	validationHeading.append('Validation');
	
	
	var borderDiv = document.createElement('div');
	root.append(borderDiv);
	borderDiv.setAttribute('class', 'list-group');
	
	var borderItemDiv = document.createElement('div');
	borderDiv.append(borderItemDiv);
	borderItemDiv.setAttribute('class', 'list-group-item border-secondary');
	
	borderItemDiv.append('Upload validation configuration files to start a validation process');

	var configForm = document.createElement('form');
	borderItemDiv.append(configForm);
	
	var configFileInput = document.createElement('input');
	configForm.append(configFileInput);
	configFileInput.setAttribute('name', 'file');
	configFileInput.setAttribute('type', 'file');
	
	var configSubmit = document.createElement('button');
	configForm.append(configSubmit);
	configSubmit.setAttribute('type', 'submit');
	configSubmit.setAttribute('class', 'btn btn-primary');
	configSubmit.append('Upload');
	
	var configCreate = document.createElement('button');
	configForm.append(configCreate);
	configCreate.setAttribute('type', 'button');
	configCreate.setAttribute('class', 'btn btn-secondary float-right');
	configCreate.setAttribute('data-toggle', 'modal');
	configCreate.setAttribute('data-target', '#validation_plan_modal');
	configCreate.setAttribute('data-id', model.id);
	configCreate.append('Create');
	
	var validationItemsDiv = document.createElement('div');
	borderItemDiv.append(validationItemsDiv);
	validationItemsDiv.setAttribute('class', 'list-group mt-3');
	
	$(configForm).submit(function (e) {
		var url = '/CityDoctorWebService/rest/models/' + model.id + '/validations';
		e.preventDefault();
		
		var data = new FormData();
		$.each($(configFileInput)[0].files, function(i, file) {
		    data.append('file', file);
		});
		
		$.ajax({
			method : 'POST',
			url : url,
		    cache: false,
		    contentType: false,
		    processData: false,
			data : data,
			beforeSend : function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + token);
			},
			success : function (response) {
				addValidationControls(validationItemsDiv, response, model);
			},
			complete : function (xhr, status) {
				if (xhr.status == 403) {
					console.log('Authentication failed')
				} else if (xhr.status == 500) {
					document.location.href = 'internal_server_error.html';
				}
			}
		});
	});
	
	model.validations.forEach(function (validation, i) {
		addValidationControls(validationItemsDiv, validation, model);
	});
}

function addValidationControls(root, validation, model) {
	var validationItemDiv = document.createElement('div');
	root.append(validationItemDiv);
	validationItemDiv.setAttribute('class', 'list-group-item border-secondary');
	
	var itemHeading = document.createElement('h5');
	validationItemDiv.append(itemHeading);
	itemHeading.append(validation.name);
	
	var badgeSpan = document.createElement('span');
	itemHeading.append(badgeSpan);
	if (validation.status == 'PARSING') {
		badgeSpan.setAttribute('class', 'badge badge-warning float-right');
		badgeSpan.append('Parsing');
		window.setTimeout(function() {
			validationStatusListener(validation, validationItemDiv, badgeSpan, model);
		}, 1000);
	} else if (validation.status == 'VALIDATING') {
		badgeSpan.setAttribute('class', 'badge badge-warning float-right');
		badgeSpan.append('Validating');
		window.setTimeout(function() {
			validationStatusListener(validation, validationItemDiv, badgeSpan, model);
		}, 1000);
	} else if (validation.status == 'OK') {
		badgeSpan.setAttribute('class', 'badge badge-success float-right');
		badgeSpan.append('Validation Successful');
		addOKValidationControls(validationItemDiv, model, validation);
	} else if (validation.status == 'ERROR') {
		badgeSpan.setAttribute('class', 'badge badge-danger float-right');
		badgeSpan.append('Validation Failed');
		addValidationDeleteButton(validationItemDiv, validation);
	}
}

function addOKValidationControls(root, model, validation) {
	var viewItemButton = document.createElement('button');
	root.append(viewItemButton);
	viewItemButton.setAttribute('class', 'btn btn-primary');
	viewItemButton.append('View');
	viewItemButton.addEventListener('click', function(e) {
		$('#modelsPage').slideToggle();
		$('#viewPage').slideToggle();
		loadBuildings(model.id, 0, validation.id);
	});
	
	addValidationDeleteButton(root, validation);
	
	var downloadDiv = document.createElement('div');
	root.append(downloadDiv);
	downloadDiv.append('Download Reports:');
	
	var xmlDownload = document.createElement('button');
	downloadDiv.append(xmlDownload);
	xmlDownload.setAttribute('class', 'ml-3 btn btn-secondary');
	xmlDownload.append('XML');
	$(xmlDownload).click(function(e) {
		downloadFile('/CityDoctorWebService/rest/models/validations/' + validation.id + '/xml');
	});
	
	var pdfDownload = document.createElement('button');
	downloadDiv.append(pdfDownload);
	pdfDownload.setAttribute('class', 'ml-1 btn btn-secondary');
	pdfDownload.append('PDF');
	$(pdfDownload).click(function(e) {
		downloadFile('/CityDoctorWebService/rest/models/validations/' + validation.id + '/pdf');
	});
	
}

function addValidationDeleteButton(root, validation) {
	var deleteItemButton = document.createElement('button');
	root.append(deleteItemButton);
	deleteItemButton.setAttribute('class', 'btn btn-danger float-right');
	deleteItemButton.append('Delete');
	deleteItemButton.addEventListener('click', function(e) {
		$.ajax({
			method : 'DELETE',
			url : '/CityDoctorWebService/rest/models/validations/' + validation.id,
			beforeSend : function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + token);
			},
			success : function(response) {
				root.parentNode.removeChild(root);
			},
			complete : function(xhr, status) {
				if (xhr.status == 403) {
					document.location.href = 'index.html';
				} else if (xhr.status == 500) {
					document.location.href = 'internal_server_error.html';
				}
			}
		});
	});
}

function loadBuildings(modelId, from, validationId) {
	currentValidationId = validationId;
	currentModelId = modelId;
	// view page is now visible, resize gl viewport
	resizeCanvas();
	
	// clear errors
	var errorList = $('#error_list');
	errorList.empty();
	var e = document.getElementById('error_heading');
	if (validationId) {
		e.style.visibility = 'visible';
	} else {
		e.style.visibility = 'hidden';
	}
	
	var buildingsReq = new XMLHttpRequest();
	if (validationId) {
		if (onlyErrors) {
			buildingsReq.open('GET', '/CityDoctorWebService/rest/models/' + modelId + '/error_buildings?from=' + from + '&valid=' + validationId);
		} else {
			buildingsReq.open('GET', '/CityDoctorWebService/rest/models/' + modelId + '/buildings?from=' + from + '&valid=' + validationId);
		}
	} else {
		buildingsReq.open('GET', '/CityDoctorWebService/rest/models/' + modelId + '/buildings?from=' + from);
	}
	buildingsReq.responseType = 'json';
	buildingsReq.setRequestHeader('Authorization', 'Bearer ' + token);
	
	buildingsReq.onload = function(e) {
		if (this.status == 200) {
			var buildings = buildingsReq.response;
			
			// setup pagination
			var pagesDiv = $('#pages');
			pagesDiv.empty();
			var maxPages = Math.trunc(buildings.to / itemsPerPage) + 1;
			if (maxPages > 1) {
				var currentPage = Math.trunc(buildings.from / itemsPerPage) + 1;
				var minPage = Math.max(1, currentPage - Math.max(2, currentPage - maxPages + (nrOfPages - 1)));
				var maxPage = Math.min(maxPages, currentPage + (nrOfPages - 1) - (currentPage - minPage));
				
				var nav = document.createElement('nav');
				pagesDiv.append(nav);
				nav.setAttribute('aria-label', 'feature navigation');
				
				var ul = document.createElement('ul');
				nav.append(ul);
				ul.setAttribute('class', 'pagination');
				
				createPageButton(ul, modelId, validationId, 0, 'First', false);
				var pageIndex;
				for (pageIndex = minPage; pageIndex < maxPage + 1; pageIndex++) {
					createPageButton(ul, modelId, validationId, (pageIndex - 1) * itemsPerPage, pageIndex, pageIndex === currentPage);
				}
				createPageButton(ul, modelId, validationId, (maxPages - 1) * itemsPerPage, 'Last', false);
			}
			var buildingDiv = $('#bu');
			buildingDiv.empty();
			buildings.buildings.forEach(function (building, i) {
				
				var buildingLabel = document.createElement('label');
				buildingDiv.append(buildingLabel);
				if (validationId) {
					if (building.hasErrors) {
						buildingLabel.setAttribute('class', 'text-left btn btn-sm btn-outline-danger');
					} else {
						buildingLabel.setAttribute('class', 'text-left btn btn-sm btn-outline-success');
					}
				} else {
					buildingLabel.setAttribute('class', 'text-left btn btn-sm btn-outline-primary');
				}
				
				var buildingRadio = document.createElement('input');
				buildingLabel.append(buildingRadio);
				buildingRadio.setAttribute('type', 'radio');
				buildingRadio.setAttribute('style', 'display: none;');
				
				buildingLabel.append(building.gmlId);
				
				buildingLabel.addEventListener('click', function (e) {
					// get geometry data
					var oReq = new XMLHttpRequest();
					oReq.open('GET', '/CityDoctorWebService/rest/buildings/' + building.id, true);
					oReq.responseType = 'arraybuffer';
					oReq.setRequestHeader('Authorization', 'Bearer ' + token);
					
					oReq.onload = function(oEvent) {
						if (this.status == 200) {
							  var arrayBuffer = oReq.response;
								
							  // accessing the received bytes
							  var nrOfVertices = arrayBuffer.byteLength / (9 * 4);
							  var dataView = new DataView(arrayBuffer);
							  var floatArray = new Float32Array(nrOfVertices * 9);
							  var i;
							  for (i = 0; i < nrOfVertices * 9; i++) {
								  floatArray[i] = dataView.getFloat32(i * 4);
							  }
							  drawModel(nrOfVertices, floatArray, building.width);
						} else if (this.status == 403) {
							window.location.href = 'index.html';
						}
					};
					oReq.send();
					
					if (validationId) {
						// get errors
						var errorReq = new XMLHttpRequest();
						errorReq.open('GET', '/CityDoctorWebService/rest/buildings/' + building.id + '/validation/' + validationId + '/errors', true);
						errorReq.responseType = 'json';
						errorReq.setRequestHeader('Authorization', 'Bearer ' + token);
						
						errorReq.onload = function(oEvent) {
							if (this.status == 200) {
								errorList.empty();
								var errors = errorReq.response;
								if (errors.length) {
									errors.forEach(function (err) {
										 var errBtn = document.createElement('button');
										 errorList.append(errBtn);
										 errBtn.setAttribute('class', 'list-group-item list-group-item-action');
										 errBtn.append(err.name);
									});
								}
							} else if (this.status == 403) {
								window.location.href = 'index.html';
							}
						};
						errorReq.send();
					} 
				});
			});
		} else if (this.status == 403) {
			window.location.href = 'index.html';
		} else if (this.status == 500) {
			window.location.href = 'internal_server_error.html';
		}
	}
	
	buildingsReq.send();
}

function setupUploadButton(token) {
	$('#uploadForm').submit(function(e) {

		$('#addCollapse').toggleClass('collapse');
		var url = '/CityDoctorWebService/rest/models';
		e.preventDefault();

		var data = new FormData();
		jQuery.each(jQuery('#file')[0].files, function(i, file) {
			data.append('file', file);
		});
		jQuery.each(jQuery('#configFile')[0].files, function(i, file) {
			data.append('configFile', file);
		});

		$.ajax({
			method : 'POST',
			url : url,
			data : data,
			contentType : false,
			processData : false,
			cache : false,
			beforeSend : function(xhr) {
				xhr.setRequestHeader('Authorization', 'Bearer ' + token);
			},
			success : function(response) {
				refreshModels(token);
			},
			complete : function(xhr, status) {
				if (xhr.status == 403) {
					document.location.href = 'index.html';
				} else if (xhr.status == 500) {
					document.location.href = 'internal_server_error.html';
				}
			}
		});
	});
}

function createPageButton(root, modelId, validationId, from, text, active) {
	var first = document.createElement('li');
	root.append(first);
	if (active) {
		first.setAttribute('class', 'page-item active');
	} else {
		first.setAttribute('class', 'page-item');
	}
	first.onclick = function(e) {
		loadBuildings(modelId, from, validationId);
	}
	
	var firstA = document.createElement('a');
	first.append(firstA);
	firstA.setAttribute('class', 'page-link');
	firstA.append(text);
}

function createFileNameListener() {
	$(".custom-file-input").on(
			"change",
			function() {
				var fileName = $(this).val().split("\\").pop();
				$(this).siblings(".custom-file-label").addClass("selected")
						.html(fileName);
			});
}

function createToggleListener() {
	var addBtn = $('#addBtn');
	addBtn.click(function() {
		addBtn.toggleClass('active');
	});
}
